@@ -3,19 +3,21 @@
 """
 import os
 import re
+from io import BytesIO
+
 import requests
 import sparql_dataframe
 
-
 try:
     from typing import Literal, get_args
 except ImportError:
     from typing_extensions import Literal, get_args
-from bs4 import BeautifulSoup
-from pdfminer.high_level import extract_text
+
 import pandas as pd
-from halo import Halo
+from bs4 import BeautifulSoup
 from fire import Fire
+from halo import Halo
+from pdfminer.high_level import extract_text
 
 
 class Eurlex:
@@ -195,7 +197,10 @@ class Eurlex:
             assert (
                 len(manual_type) > 2
             ), f"{manual_type} is invalid - please specify a proper type from http://publications.europa.eu/resource/authority/resource-type"
-        if resource_type in ["caselaw", "manual", "any"] and include_court_procedure:
+        if (
+            not resource_type in ["caselaw", "manual", "any"]
+            and include_court_procedure
+        ):
             raise Exception(
                 "Resource and variable requested are incompatible"
             )  # improve exception handling
@@ -288,7 +293,7 @@ class Eurlex:
             query += (
                 """?work cdm:resource_legal_id_sector ?sector.
                     FILTER(str(?sector)='"""
-                + sector
+                + str(sector)
                 + "')"
             )
         if resource_type == "directive":
@@ -397,7 +402,11 @@ class Eurlex:
         if resource_type == "national_implementation":
             query += """ FILTER(?type=<http://publications.europa.eu/resource/authority/resource-type/MEAS_NATION_IMPL>)"""
         if resource_type == "manual" and manual_type and len(manual_type) > 1:
-            query += """ FILTER(?type=<http://publications.europa.eu/resource/authority/resource-type/" + manual_type + ">)"""
+            query += (
+                """ FILTER(?type=<http://publications.europa.eu/resource/authority/resource-type/"""
+                + manual_type
+                + """>)"""
+            )
         if include_corrigenda is False and resource_type != "caselaw":
             query += """ FILTER not exists{?work cdm:work_has_resource-type <http://publications.europa.eu/resource/authority/resource-type/CORRIGENDUM>}"""
         if include_celex:
@@ -695,7 +704,7 @@ class Eurlex:
                     print("The CELEX url is: {}".format(url))
 
         if data_type == "title":
-            out = ""
+            out = ""  # should be dict but then should be changed in whole function
             try:
                 if __name__ == "__main__":
                     print("Getting title data...")
@@ -708,25 +717,49 @@ class Eurlex:
                 )
             except Exception as e:
                 print("There was an error during data retrieval: {}", e)
+                out = {
+                    "title": str(response.status_code),
+                    "parties": str(response.status_code),
+                    "case_number": str(response.status_code),
+                }
             if response.status_code == 200:
                 html = BeautifulSoup(response.text, "xml")
                 out = str(html.find("EXPRESSION_TITLE").get_text())
-                if extract_caselaw_metadata and "#" in out:
-                    title = out.split("#")[0].strip()
-                    parties = out.split("#")[1].strip().strip(".")
-                    case_number = out.split("#")[2].strip().strip(".")
 
-                    out = {
-                        "title": title,
-                        "parties": parties,
-                        "case_number": case_number,
-                    }
+                if extract_caselaw_metadata:
+                    if __name__ == "__main__":
+                        print("Extracting metadata...")
+                    if re.match(
+                        "(?s).*#.*#.*$",
+                        out,  # added (?s) singleline flag to also match newlines which are sometimes a part of the text
+                    ):  # "#" in out: TODO - could improve by trying to match sub-parts based on keywords or structure to their relevant parts, even when there are only 2
+                        if __name__ == "__main__":
+                            print("Extracting metadata after matching #")
+                        title = out.split("#")[0].strip()
+                        parties = out.split("#")[1].strip().strip(".")
+                        case_number = out.split("#")[2].strip().strip(".")
+
+                        out = {
+                            "title": title,
+                            "parties": parties,
+                            "case_number": case_number,
+                        }
+                    else:
+                        if __name__ == "__main__":
+                            print("No caselaw metadata extracted")
+                        out = {"title": out, "parties": "NaN", "case_number": "NaN"}
+                else:
+                    out = {"title": out, "parties": "NaN", "case_number": "NaN"}
                 if __name__ == "__main__":
                     print(out)
             else:
                 if __name__ == "__main__":
                     print("No content retrieved: {}", response.status_code)
-                out += str(response.status_code)
+                out = {
+                    "title": str(response.status_code),
+                    "parties": str(response.status_code),
+                    "case_number": str(response.status_code),
+                }
 
         elif data_type == "text":
             out = ""
@@ -755,7 +788,8 @@ class Eurlex:
                 links = []
                 for link in links_html:
                     links.append(link["href"])
-                print("Found multiple links: {}", links)
+                if __name__ == "__main__":
+                    print("Found multiple links: {}", links)
                 multiout = ""
                 for link in links:
                     multiresponse = requests.get(
@@ -767,15 +801,17 @@ class Eurlex:
                         },
                     )
                     if multiresponse.status_code == 200:
-                        print(str(multiresponse.status_code))
-                        print(multiresponse.text)
+                        if __name__ == "__main__":
+                            print(str(multiresponse.status_code))
+                            print(multiresponse.text)
                         multiout += (
                             self.read_data(multiresponse) + "---documentbreak---"
                         )
                     else:
                         multiout += "NaN"
-                print(multiout)
-                out = multiout
+                if __name__ == "__main__":
+                    print(multiout)
+                out = multiout  # TODO stringify?
             elif response.status_code == 406:
                 out += "NaN" + str(
                     response.status_code
@@ -859,24 +895,128 @@ class Eurlex:
         """
         # check content type to be html?
         content_type = response.headers.get("Content-Type")
-        if "text/html" in content_type:
+        if "text/html" in content_type or "application/xhtml" in content_type:
             html = BeautifulSoup(response.content, "html.parser")
             ret = html.find("body").get_text()
-            return ret + "---pagebreak---"
+            return ret + "---pagebreak---"  # TODO when is this really needed?
         elif "application/pdf" in content_type:
-            text = extract_text(response.content)
+            text = extract_text(BytesIO(response.content))
             return text + "---pagebreak---"
         elif "application/msword" in content_type:
             # would probably use python-docx to implement this
             ret = "The Word format is not suppported at present"
-            print(ret)
+            if __name__ == "__main__":
+                print(ret)
             return ret
         # len('Error: unsupported content type: application/xhtml+xml;charset=UTF-8')
         else:
             ret = f"Error: unsupported content type: {content_type}"
-            print(ret)
+            if __name__ == "__main__":
+                print(ret)
             return ret
 
+    "Parse curia lists"
+
+    def parse_curia(self, case_lists: list = "all", limit=None):
+        # def curia_cases(self, case_lists="all", parse=True):
+        """
+        Harvests data from lists of EU court cases from curia.europa.eu.
+        CELEX identifiers are extracted from hyperlinks where available.
+
+        Parameters:
+        -----------
+        case_lists
+            Data to be scraped from four separate lists of cases maintained by Curia, defaults to "all"
+            which contains cases from Court of Justice, General Court and Civil Service Tribunal.
+            parse. If True, references to cases and appeals are parsed out from `case_info` into separate columns
+
+        @return
+        A data frame containing case identifiers and information as character columns. Where the case id
+        contains a hyperlink to Eur-Lex, the CELEX identifier is retrieved as well.
+        """
+        print("Selected case lists are {}".format(str(case_lists)))
+        url_ecj_old = "https://curia.europa.eu/en/content/juris/c1_juris.htm"
+        url_ecj_new = "https://curia.europa.eu/en/content/juris/c2_juris.htm"
+        url_gc_all = "https://curia.europa.eu/en/content/juris/t2_juris.htm"
+        url_cst_all = "https://curia.europa.eu/en/content/juris/f1_juris.htm"
+        # Define lists of cases to be scraped
+        valid_lists = ["ecj", "gc", "cst"]
+        # TODO make into Literal properly - valid_lists = Literal["ecj", "gc", "cst"]
+        scrape_urls = []
+        # for l in case_lists:
+        if "ecj" in case_lists or case_lists == "all":
+            scrape_urls.append(url_ecj_old)
+            scrape_urls.append(url_ecj_new)
+        elif "gc" in case_lists or case_lists == "all":
+            scrape_urls.append(url_gc_all)
+        elif "cst" in case_lists or case_lists == "all":
+            scrape_urls.append(url_cst_all)
+        else:
+            print("You should not be here")
+        return self.curia_scraper(scrape_urls, limit)
+
+    def curia_scraper(self, urls, limit):
+        multiple_lists = {}
+        for u in urls:
+            # response = requests.get(u)
+            response = requests.get(u)
+            soup = BeautifulSoup(response.text, "html.parser")
+            table = soup.find("table").findAll(
+                "tr",
+            )
+            records = {}
+            for index in range(0, len(table)):
+                if 1 <= index <= limit:
+                    records[index] = {}
+                    for td in table[index]:
+                        try:
+                            records[index]["case_number"] = td.find("a")["name"]
+                        except:
+                            pass
+                        try:
+                            records[index]["case_info"] = td.find("i").text
+                        except:
+                            pass
+                        try:
+                            # print(td.find("a").next_element.a["href"])
+                            records[index]["link"] = td.find("a").next_element.a[
+                                "href"
+                            ][24:-19]
+                        except:
+                            pass
+                        try:
+                            ecli = re.search(
+                                r"^.+?(ECLI\:EU\:\w\:[0-9]{4}:[0-9]).*?$",
+                                records[index]["case_info"],
+                            )
+                            records[index]["ecli"] = ecli.group(1)
+                        except:
+                            pass
+                        try:
+                            celex = re.search(
+                                r"^http.+?CELEX.+numdoc\=(\w+?)$",
+                                records[index]["link"],
+                            )
+                            records[index]["celex"] = celex.group(1)
+                        except:
+                            pass
+                        try:
+                            curia_docs = requests.get(records[index]["link"])
+                        except:
+                            if __name__ == "__main__":
+                                print(
+                                    f"There was an error retrieving the document: {index}"
+                                )
+                        try:
+                            curia_html = BeautifulSoup(curia_docs.text, "html.parser")
+                            records[index]["case_text"] = curia_html.find(
+                                "div", attrs={"id": "TexteOnly"}, recursive=True
+                            ).text
+                        except:
+                            pass
+            multiple_lists[u] = records
+        return multiple_lists
+
 
 # The main function. It uses the fire framework to expose the functions of the module on the command line
 def main(argv=None):
